"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _isString = require("lodash/isString");

var _isString2 = _interopRequireDefault(_isString);

var _omit = require("../omit");

var _omit2 = _interopRequireDefault(_omit);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function PgConnectionArgOrderBy(builder) {
  builder.hook("init", (_, {
    newWithHooks,
    pgIntrospectionResultsByKind: introspectionResultsByKind,
    graphql: { GraphQLEnumType },
    inflection
  }) => {
    introspectionResultsByKind.class.filter(table => table.isSelectable && !(0, _omit2.default)(table, "order")).filter(table => !!table.namespace).forEach(table => {
      const tableTypeName = inflection.tableType(table);
      /* const TableOrderByType = */
      newWithHooks(GraphQLEnumType, {
        name: inflection.orderByType(tableTypeName),
        description: `Methods to use when ordering \`${tableTypeName}\`.`,
        values: {
          NATURAL: {
            value: {
              alias: null,
              specs: []
            }
          }
        }
      }, {
        pgIntrospection: table,
        isPgRowSortEnum: true
      });
    });
    return _;
  });
  builder.hook("GraphQLObjectType:fields:field:args", (args, {
    extend,
    getTypeByName,
    pgGetGqlTypeByTypeId,
    pgSql: sql,
    graphql: { GraphQLList, GraphQLNonNull },
    inflection
  }, {
    scope: {
      isPgFieldConnection,
      isPgFieldSimpleCollection,
      pgFieldIntrospection: table
    },
    addArgDataGenerator,
    Self,
    field
  }) => {
    const shouldAddOrderBy = isPgFieldConnection || isPgFieldSimpleCollection;
    if (!shouldAddOrderBy || !table || table.kind !== "class" || !table.namespace || !table.isSelectable || (0, _omit2.default)(table, "order")) {
      return args;
    }
    const TableType = pgGetGqlTypeByTypeId(table.type.id);
    const tableTypeName = TableType.name;
    const TableOrderByType = getTypeByName(inflection.orderByType(tableTypeName));
    const cursorPrefixFromOrderBy = orderBy => {
      if (orderBy) {
        let cursorPrefixes = [];
        for (const item of orderBy) {
          if (item.alias) {
            cursorPrefixes.push(sql.literal(item.alias));
          }
        }
        if (cursorPrefixes.length > 0) {
          return cursorPrefixes;
        }
      }
      return null;
    };

    addArgDataGenerator(function connectionOrderBy({ orderBy: rawOrderBy }) {
      const orderBy = rawOrderBy ? Array.isArray(rawOrderBy) ? rawOrderBy : [rawOrderBy] : null;
      return {
        pgCursorPrefix: cursorPrefixFromOrderBy(orderBy),
        pgQuery: queryBuilder => {
          if (orderBy != null) {
            orderBy.forEach(item => {
              const { specs, unique } = item;
              const orders = Array.isArray(specs[0]) || specs.length === 0 ? specs : [specs];
              orders.forEach(([col, ascending]) => {
                const expr = (0, _isString2.default)(col) ? sql.fragment`${queryBuilder.getTableAlias()}.${sql.identifier(col)}` : col;
                queryBuilder.orderBy(expr, ascending);
              });
              if (unique) {
                queryBuilder.setOrderIsUnique();
              }
            });
          }
        }
      };
    });

    return extend(args, {
      orderBy: {
        description: `The method to use when ordering \`${tableTypeName}\`.`,
        type: new GraphQLList(new GraphQLNonNull(TableOrderByType))
      }
    }, `Adding 'orderBy' to field '${field.name}' of '${Self.name}'`);
  });
};
//# sourceMappingURL=PgConnectionArgOrderBy.js.map